"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TaskRunCommand = exports.TaskRun = void 0;
const TaskRunner_1 = require("../Tasks/TaskRunner");
const TaskConfigLoader_1 = require("../Tasks/TaskConfigLoader");
const TaskConfigNotFound_1 = require("../Errors/TaskConfigNotFound");
function resolveEnvVariables(env) {
    return env
        .map((envstr) => {
        const parts = envstr.split('=');
        const name = parts[0];
        const value = parts[1];
        return [name, value];
    })
        .reduce((result, [name, value]) => {
        result[name] = value;
        return result;
    }, {});
}
class TaskRun {
    constructor() {
        this.aliases = ['run', 'r'];
        this.command = '$0 <name>';
        this.builder = {
            'dry-run': {
                alias: 'd',
                default: false,
                type: 'boolean',
            },
            env: {
                alias: 'e',
                array: true,
                default: [],
                type: 'string',
            },
            ignored: {
                alias: 'i',
                array: true,
                default: ['node_modules'],
                type: 'string',
            },
        };
        this.handler = (args) => __awaiter(this, void 0, void 0, function* () {
            const task = yield TaskConfigLoader_1.taskConfigLoader(args.cwd);
            if (task === null) {
                throw new TaskConfigNotFound_1.TaskConfigNotFound();
            }
            const entries = task.getStepEntries(args.name);
            const env = resolveEnvVariables(args.env);
            if (args.dryRun) {
                return entries.map((entry) => console.log('execute', [entry.name, ...entry.args].join(' ')));
            }
            if (args.name) {
                const results = yield TaskRunner_1.TaskRunner.from(task, args.name, { env, cwd: args.cwd });
                const failed = results.filter((result) => result.exitCode !== 0);
                failed.map((result) => console.error(result.exitCode));
                return;
            }
            throw new Error('must provide a top-level name to execute');
        });
    }
}
exports.TaskRun = TaskRun;
exports.TaskRunCommand = new TaskRun();
//# sourceMappingURL=TaskRun.js.map