"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TaskSortCommand = exports.TaskSort = void 0;
const fs_1 = require("@nofrills/fs");
const FileSort_1 = require("../Sort/FileSort");
class TaskSort {
    constructor() {
        this.aliases = ['sort', 's'];
        this.command = 'sort <glob> [format]';
        this.builder = {
            'dry-run': {
                alias: 'd',
                default: false,
                type: 'boolean',
            },
            ignored: {
                alias: 'i',
                array: true,
                default: ['node_modules', 'package.json', 'package-lock.json'],
                type: 'string',
            },
            'sort-array-properties': {
                boolean: true,
                default: false,
                type: 'boolean',
            },
        };
        this.handler = (args) => __awaiter(this, void 0, void 0, function* () {
            const files = yield fs_1.fs.glob(`${args.cwd}/${args.glob}`);
            const results = yield FileSort_1.Sorters.sort(files, args);
            const errors = results.filter((result) => result.error);
            const modified = results.filter((result) => result.error === undefined);
            if (errors.length > 0) {
                return console.error(...errors);
            }
            return modified.map((x) => console.log(args.dryRun ? '[dry-run]' : '', x.filename));
        });
    }
}
exports.TaskSort = TaskSort;
exports.TaskSortCommand = new TaskSort();
//# sourceMappingURL=TaskSort.js.map