"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Sorters = void 0;
const common_1 = require("@nnode/common");
const promise_parallel_throttle_1 = require("promise-parallel-throttle");
const SORTERS = new Set();
const DefaultSortOptions = {};
var Sorters;
(function (Sorters) {
    function register(sorter) {
        SORTERS.add(sorter);
    }
    Sorters.register = register;
    function unregister(sorter) {
        SORTERS.delete(sorter);
    }
    Sorters.unregister = unregister;
    function sorter(filename) {
        return {
            filename,
            sorters: Array.from(SORTERS.values()).filter((sorter) => sorter.sortable(filename)),
        };
    }
    Sorters.sorter = sorter;
    function sorters(filenames, options) {
        return filenames.reduce((results, filename) => {
            const ignored = options.ignored
                .map((expression) => new RegExp(expression, 'i'))
                .some((regex) => regex.test(filename));
            if (ignored) {
                return results;
            }
            return [...results, sorter(filename)];
        }, []);
    }
    Sorters.sorters = sorters;
    function sort(filenames, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const opts = common_1.Merge(DefaultSortOptions, options);
            const sortables = sorters(filenames, opts);
            const tasks = sortables.reduce((results, sortable) => {
                return [...results, ...sortable.sorters.map((x) => () => x.sort(sortable.filename, opts))];
            }, []);
            const results = yield promise_parallel_throttle_1.all(tasks);
            return results.reduce((errors, current) => {
                if (current) {
                    return [...errors, current];
                }
                return errors;
            }, []);
        });
    }
    Sorters.sort = sort;
})(Sorters = exports.Sorters || (exports.Sorters = {}));
//# sourceMappingURL=FileSort.js.map