"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.JsonSorter = void 0;
const fs_1 = require("@nofrills/fs");
const FileSort_1 = require("../FileSort");
const Logging_1 = require("../../Logging");
const ObjectSort_1 = require("../ObjectSort");
const PATTERN = /\.json$/g;
const logger = Logging_1.Logger.extend('json-sorter');
exports.JsonSorter = {
    sort: (filename, options) => __awaiter(void 0, void 0, void 0, function* () {
        try {
            const json = yield fs_1.fs.json(filename);
            const sorted = ObjectSort_1.objectSort(json, options);
            const buffer = Buffer.from(JSON.stringify(sorted, null, 2));
            if (options.dryRun === false) {
                yield fs_1.fs.writeFile(filename, buffer);
            }
            return { filename, object: sorted };
        }
        catch (error) {
            logger.error(error);
            return { error, filename, object: null };
        }
    }),
    sortable: (filename) => {
        return PATTERN.test(filename);
    },
};
FileSort_1.Sorters.register(exports.JsonSorter);
//# sourceMappingURL=JsonSorter.js.map