"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Shebang = void 0;
const fs_1 = require("@nofrills/fs");
class Shebang {
    static from(cwd) {
        return new Shebang().shebang(cwd);
    }
    shebang(cwd) {
        return __awaiter(this, void 0, void 0, function* () {
            const filename = fs_1.fs.join(cwd, 'package.json');
            const npm = yield fs_1.fs.json(filename);
            if (typeof npm.bin === 'string') {
                yield Shebang.shebangify(fs_1.fs.basename(npm.bin, false), npm.bin);
            }
            if (typeof npm.bin === 'object') {
                const bin = npm.bin;
                yield Promise.all(Object.keys(bin).map((key) => __awaiter(this, void 0, void 0, function* () {
                    try {
                        return yield Shebang.shebangify(key, fs_1.fs.join(process.cwd(), bin[key]));
                    }
                    catch (error) {
                        throw error;
                    }
                })));
            }
        });
    }
    static shebangify(name, filename) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const original = yield fs_1.fs.readFile(filename);
                if (original.toString().startsWith('#!') === false) {
                    const shebang = Buffer.from('#!/usr/bin/env node\n');
                    const combined = Buffer.concat([shebang, original]);
                    yield fs_1.fs.writeFile(filename, combined);
                    return combined;
                }
                return original;
            }
            catch (error) {
                throw error;
            }
        });
    }
}
exports.Shebang = Shebang;
//# sourceMappingURL=Shebang.js.map