"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TaskBuilder = void 0;
const TaskEntryType_1 = require("../Models/TaskEntryType");
class TaskBuilder {
    static from(config) {
        const builder = new TaskBuilder();
        return builder.transform(config);
    }
    expand(config, value) {
        if (Array.isArray(value)) {
            return this.fromArray(config, value);
        }
        if (typeof value === 'string') {
            return [this.createEntry(value)];
        }
        return Object.assign({}, value, this.expand(config, value));
    }
    fromArray(config, definitions) {
        return definitions
            .map((task) => {
            if (typeof task === 'string') {
                return this.fromString(config, task);
            }
            return [task];
        })
            .reduce((previous, current) => previous.concat(current), []);
    }
    fromString(config, command) {
        const regex = /^\[(.*)\]/g;
        const matches = regex.exec(command);
        if (matches) {
            const name = matches[1];
            const context = {
                config,
                name,
                task: this.expand(config, config.steps[name]),
            };
            return this.fromArray(context.config, context.task).map((entry) => {
                entry.origin = name;
                return entry;
            });
        }
        return [this.createEntry(command)];
    }
    createEntry(command) {
        const [cmd, type] = this.type(command);
        const parts = cmd.split(' ');
        return { args: parts.slice(1), command: parts[0], name: parts[0], type };
    }
    type(command) {
        const prefix = command[0];
        const cmd = command.slice(1);
        switch (prefix) {
            case TaskEntryType_1.TaskEntryType.env:
                return [cmd, TaskEntryType_1.TaskEntryType.env];
            case TaskEntryType_1.TaskEntryType.exec:
                return [cmd, TaskEntryType_1.TaskEntryType.exec];
            case TaskEntryType_1.TaskEntryType.nobail:
                return [cmd, TaskEntryType_1.TaskEntryType.nobail];
            case TaskEntryType_1.TaskEntryType.parallel:
                return [cmd, TaskEntryType_1.TaskEntryType.parallel];
            case TaskEntryType_1.TaskEntryType.shell:
                return [cmd, TaskEntryType_1.TaskEntryType.shell];
            case TaskEntryType_1.TaskEntryType.skip:
                return [cmd, TaskEntryType_1.TaskEntryType.skip];
            case TaskEntryType_1.TaskEntryType.spawn:
                return [cmd, TaskEntryType_1.TaskEntryType.spawn];
        }
        return [command, TaskEntryType_1.TaskEntryType.exec];
    }
    transform(config) {
        const result = Object.keys(config.steps)
            .map((key) => ({ config, name: key, task: config.steps[key] }))
            .filter((context) => (context.task ? true : false))
            .map((context) => {
            context.task = { entries: this.expand(config, context.task) };
            return context;
        })
            .reduce((result, context) => {
            result.steps[context.name] = context.task;
            return result;
        }, config);
        return result;
    }
}
exports.TaskBuilder = TaskBuilder;
//# sourceMappingURL=TaskBuilder.js.map