"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.taskConfigLoader = exports.TASK_LOADER_FILES = void 0;
const fs_1 = require("@nofrills/fs");
const Logging_1 = require("../Logging");
const TaskBuilder_1 = require("./TaskBuilder");
const TaskNavigator_1 = require("./TaskNavigator");
const TaskValidate_1 = require("./TaskValidate");
exports.TASK_LOADER_FILES = ['.tasks.json', 'tasks.json'];
const log = Logging_1.Logger.extend('loader');
function convertFromV1(task) {
    return { options: {}, steps: task.tasks };
}
function isTaskArray(value) {
    const containsTasks = typeof value === 'object';
    const containsTaskCommands = Object.keys(value).some((key) => Array.isArray(value[key]));
    return containsTasks && containsTaskCommands;
}
function fromPackage(cwd) {
    return __awaiter(this, void 0, void 0, function* () {
        const filename = fs_1.fs.join(cwd, 'package.json');
        if (yield fs_1.fs.exists(filename)) {
            const task = yield fs_1.fs.json(filename);
            log.trace('from-package', filename, task);
            if (task.tasks && isTaskArray(task.tasks)) {
                return convertFromV1(task);
            }
        }
        return null;
    });
}
function fromTaskConfig(cwd) {
    return __awaiter(this, void 0, void 0, function* () {
        return exports.TASK_LOADER_FILES.slice(1).reduce((results, file) => __awaiter(this, void 0, void 0, function* () {
            const taskfile = fs_1.fs.join(cwd, file);
            if (yield fs_1.fs.exists(taskfile)) {
                const config = yield fs_1.fs.json(taskfile);
                log.trace('from-config', taskfile, config);
                if (isTaskArray(config)) {
                    return convertFromV1({ tasks: config });
                }
                if (config.tasks) {
                    return convertFromV1(config);
                }
                if (config.steps || config.options) {
                    return config;
                }
            }
            return results;
        }), Promise.resolve(null));
    });
}
/**
 * Finds a configuration file where possible. The order in which it performs
 * the search is as follows:
 *   - looks for a .tasks.json or tasks.json
 *   - looks in package.json
 *
 * @cwd string
 * @returns @TaskV2 or @null
 */
function taskConfigLoader(cwd, validate = true) {
    return __awaiter(this, void 0, void 0, function* () {
        const taskFromConfig = yield fromTaskConfig(cwd);
        if (taskFromConfig !== null) {
            const config = TaskBuilder_1.TaskBuilder.from(taskFromConfig);
            log.trace('build-config', config);
            if (validate && (yield TaskValidate_1.taskValidate(config)) === false) {
                throw new Error('invalid configuration');
            }
            return new TaskNavigator_1.TaskNavigator(config);
        }
        const tasksFromPackage = yield fromPackage(cwd);
        if (tasksFromPackage !== null) {
            const config = TaskBuilder_1.TaskBuilder.from(tasksFromPackage);
            log.trace('build-package', config);
            if (validate && (yield TaskValidate_1.taskValidate(config)) === false) {
                throw new Error('invalid configuration');
            }
            return new TaskNavigator_1.TaskNavigator(config);
        }
        throw new Error('failed to find any suitable task configuration files');
    });
}
exports.taskConfigLoader = taskConfigLoader;
//# sourceMappingURL=TaskConfigLoader.js.map