"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TaskNavigator = void 0;
const TaskEntryType_1 = require("../Models/TaskEntryType");
const TaskValidate_1 = require("./TaskValidate");
class TaskNavigator {
    constructor(task) {
        this.task = task;
    }
    getParallelEntries(name) {
        return this.steps(this.task)
            .filter((step) => step.name === name)
            .map((step) => step.entries.filter((entry) => entry.type === TaskEntryType_1.TaskEntryType.parallel))
            .reduce((entries, entry) => entries.concat(entry), [])
            .filter((entry) => entry.type !== TaskEntryType_1.TaskEntryType.skip);
    }
    getRunnableEntries() {
        return this.steps(this.task)
            .filter((step) => step.name.includes(':') === false)
            .map((step) => step.name);
    }
    getStepEntries(name) {
        return this.steps(this.task)
            .filter((step) => step.name === name)
            .map((step) => step.entries.filter((entry) => entry.type !== TaskEntryType_1.TaskEntryType.parallel))
            .reduce((entries, entry) => entries.concat(entry), [])
            .filter((entry) => entry.type !== TaskEntryType_1.TaskEntryType.skip);
    }
    validate() {
        return TaskValidate_1.taskValidate(this.task);
    }
    steps(task) {
        return Object.keys(task.steps).map((name) => ({ name, entries: task.steps[name].entries }));
    }
}
exports.TaskNavigator = TaskNavigator;
//# sourceMappingURL=TaskNavigator.js.map