"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TaskRunner = void 0;
const os_1 = __importDefault(require("os"));
const promise_parallel_throttle_1 = require("promise-parallel-throttle");
const common_1 = require("@nnode/common");
const Logging_1 = require("../Logging");
const TaskExecutor_1 = require("./TaskExecutor");
const DefaultTaskRunnerOptions = {
    concurrency: os_1.default.cpus().length,
    cwd: process.cwd(),
    env: {},
    streaming: true,
};
class TaskRunner {
    constructor(task) {
        this.task = task;
        this.log = Logging_1.Logger.extend('task-runner');
    }
    static from(task, name, options = {}) {
        const runner = new TaskRunner(task);
        return runner.run(name, options);
    }
    run(name, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const merged = common_1.Merge(DefaultTaskRunnerOptions, options);
            this.log.trace('run', merged);
            const parallels = this.task.getParallelEntries(name).map((entry) => () => this.exec(entry, merged));
            const steps = this.task.getStepEntries(name).map((entry) => () => this.exec(entry, merged));
            this.log.trace('run-serial', steps.length, 'run-parallel', parallels.length);
            const serial = promise_parallel_throttle_1.all(steps, { maxInProgress: 1 });
            const parallel = promise_parallel_throttle_1.all(parallels, { maxInProgress: merged.concurrency });
            const serialResults = yield serial;
            const parallelResults = yield parallel;
            return serialResults.concat(parallelResults);
        });
    }
    exec(entry, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const executor = new TaskExecutor_1.TaskExecutor();
            const sub = executor.subscribe((result) => console.log(...result.stdout), (error) => console.error(error.message), () => sub.unsubscribe());
            return executor.execute(entry, options);
        });
    }
}
exports.TaskRunner = TaskRunner;
//# sourceMappingURL=TaskRunner.js.map